/*===--------------------------------------------------------------------------
 *                   ROCm Device Libraries
 *
 * This file is distributed under the University of Illinois Open Source
 * License. See LICENSE.TXT for details.
 *===------------------------------------------------------------------------*/

#include "oclc.h"

const __constant bool __oclc_wavefrontsize64 = 0;
const __constant uint __oclc_wavefrontsize_log2 = 5;
